/***********************************************************************

	File:	dumpfst.c
	Rev:	a-1
	Date:	10/22/98

	Copyright (c) 1993, 1999 by David M. Warme

************************************************************************

	Main routine for a utility to dump a set of FSTs.

************************************************************************

	Modification Log:

	a-1:	10/22/98	warme
		: Created.

************************************************************************/

#include "p1io.h"
#include "steiner.h"


/*
 * Global Routines
 */

int			main (int, char **);


/*
 * Local Routines
 */

static void		decode_params (int, char **);
static void		sortints (int *, int);
static void		usage (void);


/*
 * Local Variables
 */

static bool		aflag = FALSE;
static bool		lflag = FALSE;
static char *		me;
static bool		sflag = FALSE;

/*
 * The main routine for the fsplot utility.
 */

	int
main (

int		argc,
char **		argv
)
{
int			i;
int			j;
int			n;
int			nedges;
int			nmasks;
int *			fterms;
bitmap_t *		edge_mask;
struct cinfo		cinfo;
char			buf [128];

	setbuf (stdout, NULL);

	decode_params (argc, argv);

	init_tables ();

	read_phase_1_data (&cinfo);

	edge_mask	= cinfo.initial_edge_mask;

	nedges = cinfo.num_edges;
	nmasks = cinfo.num_edge_masks;

	for (i = 0; i < nedges; i++) {
		if ((NOT aflag) AND (NOT BITON (edge_mask, i))) continue;
		fterms = cinfo.edge [i];
		n = cinfo.edge [i + 1] - fterms;
		if (sflag) {
			sortints (fterms, n);
		}
		for (j = 0; j < n; j++) {
			printf (" %d", fterms [j]);
		}
		if (lflag) {
			dist_to_string (buf, cinfo.cost [i]);
			printf (" %s", buf);
		}
		printf ("\n");
	}

	return (0);
}

/*
 * This routine decodes the various command-line arguments.
 */

	static
	void
decode_params (

int		argc,
char **		argv
)
{
char *		ap;
char		c;

	--argc;
	me = *argv++;
	while (argc > 0) {
		ap = *argv++;
		if (*ap NE '-') {
			usage ();
		}
		++ap;
		while ((c = *ap++) NE '\0') {
			switch (c) {
			case 'a':
				aflag = TRUE;
				break;

			case 'l':
				lflag = TRUE;
				break;

			case 's':
				sflag = TRUE;
				break;

			default:
				usage ();
				break;
			}
		}
		--argc;
	}
}

/*
 * This routine prints out the proper usage and exits.
 */

static char *	arg_doc [] = {
	"",
	"\t-a\tDump all FSTs, even those marked as never used.",
	"\t-l\tInclude length of each FST.",
	"\t-s\tSorts the terminals of each FST.",
	"",
	NULL
};

	static
	void
usage (void)

{
char **		pp;
char *		p;

	(void) fprintf (stderr,
			"\nUsage: %s [-als]\n",
			me);
	pp = &arg_doc [0];
	while ((p = *pp++) NE NULL) {
		(void) fprintf (stderr, "%s\n", p);
	}
	exit (1);
}

/*
 * Sort an array of integers into increasing order.
 * We assume the array is fairly short.
 */

	static
	void
sortints (

int *		array,		/* IN/OUT - the array to sort */
int		n		/* IN - number of ints in the array */
)
{
int		i;
int		limit;
int		new_limit;
int		swapped;
int		temp;

	limit = n;
	do {
		swapped = FALSE;
		for (i = 1; i < limit; i++) {
			if (array [i-1] > array [i]) {
				temp = array [i-1];
				array [i-1] = array [i];
				array [i] = temp;
				swapped = TRUE;
				new_limit = i;
			}
		}
		limit = new_limit;
	} while (swapped);
}
