/***********************************************************************

	File:	sec_comp.h
	Rev:	a-1
	Date:	10/05/96

	Copyright (c) 1996, 1999 by David M. Warme

************************************************************************

	Data structures for decomposition to make separation of
	SEC's easier.

************************************************************************

	Modification Log:

	a-1:	10/05/96	warme
		: Created.

************************************************************************/

#ifndef SEC_COMP_H
#define	SEC_COMP_H

#include "steiner.h"


/*
 * The following structure contains all of the information that we
 * need to represent a single component of an SEC separation problem.
 * We use a variety of techniques to split one separation problem into
 * several components, and to simplify those components.
 */

struct comp {
	struct comp *	next;		/* Linked list of components */
	int		num_terms;	/* Number of terminals in component */
	int		num_fsets;	/* Number of full sets in component */
	int8u		flags;		/* See below */
	/* Info about full sets in the component... */
	int *		fsnum;		/* Original full set number */
	double *	x;		/* LP assigned weight */
	int **		fsterms;	/* Terminals in each full set */
	/* Info about terminals in the component... */
	int **		tfsets;		/* Full sets for each terminal */
	double *	tviol;		/* Extra violation for each terminal */
	bitmap_t *	tmasks;		/* Mask of original terminals that */
					/* are represented by each terminal */
					/* (we simplify by merging multiple */
					/* terminals into one...) */
	int		kmasks;		/* Size of each mask in "tmasks" */
	bitmap_t *	tmap;		/* Mask of valid terminals */
	bitmap_t *	fset_mask;	/* Mask of valid full sets */
	struct constraint * cp;		/* List of constraints for this	*/
					/* component */
};

/*
 * Various flags.  These are used to keep track of which hurdles a given
 * component has cleared.  By keeping track of this info we apply only
 * those transformations that have not previously been done to the component.
 * When a component is simplified or further divided, however, we can reset
 * these flags on the sub-component(s) -- permitting further decomposing or
 * simplification without fear of infinite looping or recursion.
 */

#define	CFLG_CONG	0x01	/* Known to contain only congested terminals */
#define CFLG_CC		0x02	/* Known to be a connected component */
#define CFLG_BCC	0x04	/* Known to be a bi-connected component */
#define	CFLG_CHAIN	0x08	/* Long integral chains merged */

#define CFLG_ALL	(CFLG_CONG | CFLG_CC | CFLG_BCC | CFLG_CHAIN)


extern struct constraint * check_component_subtour (bitmap_t *,
						    struct comp *,
						    struct constraint *,
						    double *,
						    bitmap_t *,
						    struct cinfo *);
extern struct comp *	delete_terminal_from_component (int, struct comp *);
extern struct comp *	find_congested_components (double *	x,
						   bitmap_t *	tmap,
						   bitmap_t *	fset_mask,
						   bool		print_flag,
						   struct cinfo * cip);

extern int		find_least_congested_terminal (bitmap_t *,
						       struct comp *);
extern void		free_congested_component (struct comp *);

#endif
